from __future__ import absolute_import

from ogscm.building_blocks.ccache import ccache
from ogscm.building_blocks.cppcheck import cppcheck
from ogscm.building_blocks.cvode import cvode
from ogscm.building_blocks.eigen import eigen
from ogscm.building_blocks.iwyy import iwyy
from ogscm.building_blocks.jenkins_node import jenkins_node
from ogscm.building_blocks.lmod import lmod
from ogscm.building_blocks.ogs import ogs
from ogscm.building_blocks.ogs_base import ogs_base
from ogscm.building_blocks.osu_benchmarks import osu_benchmarks
from ogscm.building_blocks.paraview import paraview
from ogscm.building_blocks.petsc import petsc
from ogscm.building_blocks.pm_conan import pm_conan
from ogscm.building_blocks.pm_easybuild import pm_easybuild
from ogscm.building_blocks.pm_spack import pm_spack
from ogscm.building_blocks.vtk import vtk
